/* $Id: guile-ext.h,v 1.4 2002/01/10 04:04:26 sgt Exp $
 * guile-ext.h
 * 
 * Extensions I hope will become a part of guile
 * Organized here by Steve Tell, with  inspiration 
 * from libguile/snarf.h and SCWM's documentation system.
 * 

 */


#ifndef GUILE_EXT_H__
#define GUILE_EXT_H__

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* three ways of making a hook, taken from SCWM's compatibility layer */
#ifndef HAVE_SCM_MAKE_HOOK
#define SCM__INIT_HOOK(var, name, nargs) do { var = scm_sysintern(name, SCM_EOL); } while (0)
#else
#ifdef HAVE_SCM_CREATE_HOOK
#define SCM__INIT_HOOK(var, name, nargs) do { var = scm_create_hook(name,nargs); } while (0)
#else
#define SCM__INIT_HOOK(var, name, nargs) do { var = scm_make_named_hook(name,nargs); } while (0)
#endif
#endif

#include <libguile/snarf.h>


/*
 * Guile 1.4 snarfing features that we rely on.  This lets us work
 * with guile-1.3.4, at least.
 */
#ifndef SCM_SNARF_HERE
#ifndef SCM_MAGIC_SNARFER
#  define SCM_SNARF_HERE(X) X
#  define SCM_SNARF_INIT(X)
#  define SCM_SNARF_DOCS(X)
#else
#  define SCM_SNARF_HERE(X)
#  define SCM_SNARF_INIT(X) SCM__I X
#  define SCM_SNARF_DOCS(X) X
#endif
#endif

#ifdef HAVE_1ARG_SCMSNARFDOC
# define GWAVE_SNARF_DOCS(TYPE, FNAME, ARGLIST, REQ, OPT, VAR, DOCSTRING)
#else
# define GWAVE_SNARF_DOCS(TYPE, FNAME, ARGLIST, REQ, OPT, VAR, DOCSTRING) \
SCM_SNARF_DOCS(TYPE, FNAME, ARGLIST, REQ, OPT, VAR, DOCSTRING)
#endif

#ifndef SCM_FUNC_CAST_ARBITRARY_ARGS
#define SCM_FUNC_CAST_ARBITRARY_ARGS SCM (*)()
#endif

#ifndef SCM_DEFINE
#define SCM_DEFINE(FNAME, PRIMNAME, REQ, OPT, VAR, ARGLIST, DOCSTRING) \
SCM_SNARF_HERE(\
static const char s_ ## FNAME [] = PRIMNAME; \
SCM FNAME ARGLIST\
)\
SCM_SNARF_INIT(\
scm_make_gsubr (s_ ## FNAME, REQ, OPT, VAR, \
                (SCM_FUNC_CAST_ARBITRARY_ARGS) FNAME); \
)\
GWAVE_SNARF_DOCS(primitive, PRIMNAME, ARGLIST, REQ, OPT, VAR, DOCSTRING)
#endif

#define SCM_HOOK(var, name, nargs, arglist, docstring) \
SCM_SNARF_HERE(static SCM var)\
SCM_SNARF_INIT( SCM__INIT_HOOK(var, name, nargs) ); \
GWAVE_SNARF_DOCS(hook, name, arglist, nargs, 0, 0, docstring)

#define SCM_GLOBAL_HOOK(var, name, nargs, arglist, docstring) \
SCM_SNARF_HERE(SCM var)\
SCM_SNARF_INIT( SCM__INIT_HOOK(var, name, nargs) ); \
GWAVE_SNARF_DOCS(hook, name, arglist, nargs, 0, 0, docstring)

#define SCM_CONCEPT(name, docstring)\
GWAVE_SNARF_DOCS(concept, name, 0, 0, 0, 0, docstring)

/* override SCM_VCELL and friends from snarf.h with our documenting
 * versions.
 */
#undef SCM_VCELL
#undef SCM_VCELL_INIT
#undef SCM_GLOBAL_VCELL
#undef SCM_GLOBAL_VCELL_INIT

#define SCM_VCELL(c_name, scheme_name, docstring) \
SCM_SNARF_HERE(static SCM c_name) \
SCM_SNARF_INIT(c_name = scm_permanent_object (scm_intern0 (scheme_name)); SCM_SETCDR (c_name, SCM_BOOL_F)); \
GWAVE_SNARF_DOCS(vcell, scheme_name, 0, 0, 0, 0, docstring)

#define SCM_GLOBAL_VCELL(c_name, scheme_name, docstring) \
SCM_SNARF_HERE(SCM c_name) \
SCM_SNARF_INIT(c_name = scm_permanent_object (scm_intern0 (scheme_name)); SCM_SETCDR (c_name, SCM_BOOL_F)); \
GWAVE_SNARF_DOCS(vcell, scheme_name, 0, 0, 0, 0, docstring)

#define SCM_VCELL_INIT(c_name, scheme_name, init_val, docstring) \
SCM_SNARF_HERE(static SCM c_name) \
SCM_SNARF_INIT(c_name = scm_permanent_object (scm_intern0 (scheme_name)); SCM_SETCDR (c_name, init_val));\
GWAVE_SNARF_DOCS(vcell, scheme_name, 0, 0, 0, 0, docstring)

#define SCM_GLOBAL_VCELL_INIT(c_name, scheme_name, init_val, docstring) \
SCM_SNARF_HERE(SCM c_name) \
SCM_SNARF_INIT(c_name = scm_permanent_object (scm_intern0 (scheme_name)); SCM_SETCDR (c_name, init_val));\
GWAVE_SNARF_DOCS(vcell, scheme_name, 0, 0, 0, 0, docstring)

#endif /* GUILE_EXT_H__ */

